# encoding: utf-8

# ------------------------------------------------------------------------------
# Copyright (c) 2016 SUSE Linux GmbH, Nuernberg, Germany.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact SUSE Linux GmbH.
#
# ------------------------------------------------------------------------------
#
# Summary: SUSE Connect Program: tests for the Product class
# Authors: Ilya Manyugin <ilya.manyugin@suse.com>

require_relative 'spec_helper'
require 'suse_connect_program/product'

describe SuseConnectProgram::Product do
  before do
    allow(File).to receive(:read).and_call_original
    allow(File).to receive(:read).with('/usr/share/YaST2/data/suse_connect_program/template_gui.erb')
      .and_return(template('template_gui.erb'))
    allow(File).to receive(:read).with('/usr/share/YaST2/data/suse_connect_program/template_ncurses.erb')
      .and_return(template('template_ncurses.erb'))
  end

  describe '#all' do
    it 'reads all entries from the file' do
      products = SuseConnectProgram::Product.all(file_path('products_single_valid.json'))
      expect(products.size).to eq 1
    end

    it 'assigns the necessary fields' do
      products = SuseConnectProgram::Product.all(file_path('products_single_valid.json'))
      item = products[0]
      expect(item.name).to eq('Product 1')
      expect(item.tech_name).to eq('product_1')
    end

    it 'filters out malformed records' do
      products = SuseConnectProgram::Product.all(file_path('products_two_malformed.json'))
      expect(products.size).to eq 1
    end

    it 'raises an exception and writes to the log when given an invalid path' do
      expect { SuseConnectProgram::Product.all(file_path('nonexistent.json')) }
        .to raise_error(RuntimeError, /Reading products file/)
      expect(log_contains?('Reading file .* failed.')).to be true
    end

    it 'sorts products first by the type, then by the name' do
      products = SuseConnectProgram::Product.all(file_path('products_four.json'))
      product_names = products.map(&:name)
      sorted_products = products.sort_by { |p| [p.type, p.name] }
      sorted_names = sorted_products.map(&:name)
      expect(product_names).to eq sorted_names
    end

    it 'throws an error if JSON is invalid' do
      expect { SuseConnectProgram::Product.all(file_path('invalid.json')) }.to raise_error(RuntimeError)
    end
  end
end
